function [global_err, class_err] = my_knn(data_a, data_t, knn, info)
% data_a - training data set used by the K-NN algorithm 
% data_t - testing  data set 
% Last column of this data set contains the class associated to feature vector 
% {x1, x2, ...., xn, class}
% When is computed the classification error (for the testing set) is necessary 
% to specify the output class.
% knn - parameter which specify the neighbors number used by kNN clasiffier
% info = 1: enable the grahics/diplay the information, 0:disable 
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE

if nargin<2, 
    %% Variant 1 - Set of clusters generated random
    %data_a = generare_date;
    %data_t = generare_date([100 40 65 70 55], [1 1; 0.2 0.2; 0.15 0.25; 0.15 0.2; 0.2 0.15], [0 0; 0.8 0.7; 0.05 0.2; 0.7 0; 0.4 0.5]);
    %% Variant 2 - IRIS data set
    load fisheriris
    clasa=strcmp(species(:),'setosa')+2*strcmp(species(:),'versicolor')+3*strcmp(species(:),'virginica');
    data = [meas, clasa]; percent = 0.5; %percent of training/testing data 
    [train, test] = crossvalind('holdOut',clasa,percent);
    data_a=data(train,:);
    data_t=data(test,:);
end
if nargin<3, knn=1; end 
if nargin<4, info=1; end 

if (size(data_a,1)==size(data_t,1)) & (sum(data_a(:))-sum(data_t(:)) <1e-10),
     same_data = 1; else same_data=0;
 end     

[len_data_t, data_dim] = size(data_t);
if len_data_t < data_dim, data_t = data_t'; [len_data_t, data_dim] = size(data_t); end

if size(data_a, 2)==data_dim, len_data_a = size(data_a, 1); else len_data_a = size(data_a, 2); end
data_dim = data_dim-1; %last column represent the class id


clasificare = zeros(len_data_t, 1); %matrix of distances to centers
for k = 1:len_data_t
    %compute distance from curent testing vector(feature) to all training vectors 
    dist = sum( (data_a(:,1:data_dim) - ones(len_data_a,data_dim) *diag(data_t(k,1:data_dim)) ).^2, 2);
    %if data_dim==1, dist = abs(data_a(:,1) - data_t(k,1) ); end % 1-D data
    
    if same_data==1
        %if the training and testing sets are the same it will not be considered the curent pattern k
        dist(k) = max(dist);
    end
    
    if knn==1
        %compute the minimum distance to the nearest point (knn=1)
        [dist_min, poz_min] = min(dist);
        %the classification is made in the class with the nearest points
        clasificare(k) = data_a(poz_min, end);
    else
        [dist_min poz_min] = sort(dist); %can be replaced with the searching of first k distances (require k crossing of distance vector)
        poz_min = poz_min(1:knn); 
        dist_min = dist_min(1:knn);
        class_min = data_a(poz_min, end); %keep the classes of nearest k points for the current vector (point)
        if sum(diff(sort(class_min)))>0 %if there are many classes
            class_id = sort(class_min);
            poz_id = [find(diff(class_id)); knn];
            class_id = class_id(poz_id); %founded classes (distinct id values)
            n_id = [poz_id(1); diff(poz_id)]; %number of occurences in each class
            candidate_classes = class_id(find(n_id==max(n_id)));
            no_candidates = length(candidate_classes);
            if no_candidates>1, % if there are two ore more than two classes with the same number of points
                % in this case compute the total distance until to all points from each clases 
                % and chose the class with a minimal total distance
                candidate_total_dist = zeros(1,no_candidates);
                for j = 1 : no_candidates
                    candidate_total_dist(j) = sum(dist_min(class_min==candidate_classes(j)));
                end
                [candidate_min_dist, poz] = min(candidate_total_dist);
                clasificare(k) = candidate_classes(poz);
            else %avem numar de puncte apropiate maxim intr-o singura clasa
                clasificare(k) = candidate_classes;
            end
        else    %all the points are situated in the same class
            clasificare(k) = class_min(1);
        end
    end
end
global_err = length( find(clasificare-data_t(:,end)) )/len_data_t;

id_class = data_t([find(diff(sort(data_t(:,end))))' len_data_t], end);
class_err = zeros(length(id_class), 1);
for i = 1 : length(id_class)
    poz = find(data_t(:,end)==id_class(i));
    class_err(i) = length( find(clasificare(poz)-data_t(poz,end)) )/length(poz);
end    

if info==1,
    disp(sprintf('Eroare de clasificare: %.2f', global_err*100));
    reprezentare_grafica_clusterizare_date(data_a, data_t, clasificare);
end
%--------------------------------------------------------------------------------
function data = generare_date(points_no, dim, d);
% add a cluster with a specified number of points in a interval of dimension 
% [x_dim y_dim] tranlates with [dx dy]
if nargin<3
    class_id=0; points_no=200; data = rand(points_no,2); data=[data ones(points_no,1)*class_id];

    class_id=1; points_no=40; x_dim=0.2; y_dim=0.2; dx=0.8; dy=0.7;
    cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
    data = [data; [cluster ones(points_no,1)*class_id]];

    class_id=2; points_no=65; x_dim=0.15; y_dim=0.25; dx=0.05; dy=0.2;
    cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
    data = [data; [cluster ones(points_no,1)*class_id]];

    class_id=3; points_no=70; x_dim=0.15; y_dim=0.2; dx=0.7; dy=0.0;
    cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
    data = [data; [cluster ones(points_no,1)*class_id]];

    class_id=4; points_no=55; x_dim=0.2; y_dim=0.15; dx=0.4; dy=0.5;
    cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
    data = [data; [cluster ones(points_no,1)*class_id]];
else
    data = [];
    class_id=0;
    for i = 1:length(points_no)
        x_dim=dim(i,1); y_dim=dim(i,2); dx=d(i,1); dy=d(i,2);
        cluster = rand(points_no(i),2)*[x_dim 0; 0 y_dim] + ones(points_no(i),2)*[dx 0; 0 dy];
        data = [data; [cluster ones(points_no(i),1)*class_id]];
        class_id = class_id+1;
    end
end
%--------------------------------------------------------------------------------
function reprezentare_grafica_clusterizare_date(data_a, data_t, clasificare)
if size(data_a,2)~=3, return; end

temp = sort(data_a(:,end));
class_id = [temp(find(diff(temp))); temp(end)];
no_clusters = length(class_id);

%markers = 'so*x+Dphv^<>'; n_markers = length(markers);
colors = 'brmkgc'; n_colors = length(colors);
for i=1:no_clusters
    index_a = find(data_a(:,end) == class_id(i));
    index_t = find(data_t(:,end) == class_id(i));
    index_c = find(clasificare == class_id(i));

    crt_color = mod(i,n_colors); if crt_color==0, crt_color=n_colors; end
    %crt_marker = mod(i,n_markers); if crt_marker==0, crt_marker=n_markers; end
    crt_color = colors(crt_color); %crt_marker = markers(crt_marker);
    
    figure(1); hold on; plot(data_a(index_a,1), data_a(index_a,2),[crt_color '.']); hold off; title('Set date de antrenare');
    figure(2); hold on; plot(data_t(index_t,1), data_t(index_t,2),[crt_color '.']); hold off; title('Set date de test');
    
    figure(3); plot(data_a(index_a,1), data_a(index_a,2),[crt_color '.']); hold on;
               plot(data_t(index_c,1), data_t(index_c,2),[crt_color 'o'], 'MarkerSize',4);
end
figure(3); legend('data_a', 'data_t'); hold off; 
figure(1); hold off; figure(2); hold off; 