function test_KNN_emotions
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE
%--------------------------------------------------------------------------

rulari_multiple=1; %0-FALSE, 1-TRUE
%mean_F0:3 mean_F1-F4:6,9,12,15 std_F0:4 std_F1-F4:7,10,13,16 median_F0:5 median_F1-F4:8,11,14,17
atr=[3:17]; %list of atributes
tip_parcurgere = 1; %1:classify acording to the emotion, 2:gender
tip_sex = 'all'; %'m'-male, 'f'-female, 'all'-all speakers
%---------------------------------
%Running parameters of KNN
percent = 0.75; %percent of date from training/testing data sets
knn = [1: 1 :50];   %value/vector of values with the number of neighbors

%---------------------------------
% Generate the training and testing data sets
if exist('DB.mat','file')==2,
%    [data, no_values] = generare_clustere_vocale(atr,'f');
    [data, no_values] = generare_clustere_emotii(atr, tip_sex, tip_parcurgere);
    if tip_parcurgere==1, data = data(:,1:end-1); end %keep the class id penultima column (id_emotie)
    if tip_parcurgere==2, data = [data(:,1:end-2) data(:,end)]; end %keep the class id as last column (id_vocala)
    
    if tip_parcurgere==1 & strcmp(tip_sex,'all')==1,
        % Emotional states 1-'BF',2-'BM',3-'FF',4-'FM',5-'TF',6-'TM',7-'NF',8-'NM' become
        % only 4 emotional states 1-'B', 3-'F', 5-'T', 7-N' (B/joy,F/Fury,T/Sadness, N/neutral tone
        poz = find(mod(data(:,end),2)==0);
        data(poz,end) = data(poz,end)-1;
    end        
else
    disp('Baza de date cu valori formantice F0, F1,...,F4 nu a fost gasita');
    data = generare_clustere;
    no_values = [0 find(diff(data(:,end)))' size(data,1)];
end    

disp(['Set atribute:' sprintf('%d ', atr)]);

if rulari_multiple==1, n_iter=50; else n_iter=1; end
err_min = zeros(1,n_iter);

for i = 1 : n_iter
    prag = round(diff(no_values)*percent); %used to select training/testing data sets
    data_a = []; %data_a = zeros(sum(prag),3);
    data_t = []; %data_t = zeros(size(data,1)-sum(prag),3);
    for j = 1 : length(no_values)-1 %length(DB_all)
        %% Selection of first 'prag(j)' values in the training set, all other values are in testing data set
        %data_a = [data_a; data(no_values(j)+1:no_values(j)+prag(j), :)];
        %data_t = [data_t; data(no_values(j)+prag(j)+1:no_values(j+1), :)];

        %% Selection of first values in the training set
        %mark = zeros(1, no_values(j+1)-no_values(j)); mark(1:prag(j))=1; 
        %% Selection of last values in the training set
        %mark = zeros(1, no_values(j+1)-no_values(j)); mark(end-prag(j)+1:end)=1;
       
        % Random selection of values from training/testing data sets
        mark = ruleta(prag(j), no_values(j+1)-no_values(j));
        data_a = [data_a; data(no_values(j)+find(mark), :)];
        data_t = [data_t; data(no_values(j)+find(~mark), :)];
     end

    err = zeros(1, length(knn));
    for k = 1:length(knn);
        [err(k) class_err(:,k)]= my_knn(data_a, data_t, knn(k),0);
        if n_iter==1,
            disp(sprintf('knn:%2d, err:%.2f',knn(k), err(k)*100));
        end
    end    
    [err_min(i), poz_min] = min(err);     class_err_min(:,i) = class_err(:,poz_min);
    
    if n_iter==1,
        disp(sprintf('Eroare minima: %.2f (knn=%d)', err_min*100, knn(poz_min) ));
    else
         s = sprintf('%d. Eroare: %.2f (knn=%2d)', i, err_min(i)*100, knn(poz_min) );
         s = [s sprintf(' | %.2f',class_err_min(:,i)*100)]; disp(s);
    end
end

disp('-----------------');
if n_iter==1,
    figure(4); plot(knn, err); title('Recognition error'); grid; axis tight;
    %Graphic representation of the best clasification provided by kNN
    my_knn(data_a(:,[1,2,end]), data_t(:,[1,2,end]), knn(poz_min),1);
else
    disp(sprintf('Eroare minima: %.2f, eroare medie: %.2f, eroare maxima: %.2f', min(err_min)*100, mean(err_min)*100, max(err_min)*100) );
    disp(['Eroare minima pentru fiecare clasa: ' sprintf(' %.2f,', min(class_err_min')*100) ]);
    disp(['Eroare medie pentru fiecare clasa: ' sprintf(' %.2f,', mean(class_err_min')*100) ]);
    disp(['Eroare maxima pentru fiecare clasa: ' sprintf(' %.2f,', max(class_err_min')*100) ]);
end

disp('gata');           
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
function mark = ruleta(n_sel, n_total)
% Selection o a number of 'n_sel' values from a total number of 'n_total' values/patterns
mark = zeros(1, n_total);
for i = 1:n_sel
    poz = fix(rand(1)*n_total)+1;
    while mark(poz)==1 | poz>n_total,
        poz = fix(rand(1)*n_total)+1;
    end
    mark(poz)=1;
end    
%--------------------------------------------------------------------------
function data = generare_clustere
% Generate a number of 100 points distributed in the hole space, and after that
% generate 4 clusters with different dimension x_dim,y_dim and deplacements/ 
% translations dx,dy.  Each point/feature vector is described by (x,y) coordinates of points

data = rand(100,2);
class = zeros(100,1);
% First cluster
points_no=40; x_dim=0.2; y_dim=0.2; dx=0.8; dy=0.7;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*1];
% Second cluster
points_no=65; x_dim=0.15; y_dim=0.25; dx=0.05; dy=0.2;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*2];

points_no=70; x_dim=0.15; y_dim=0.2; dx=0.7; dy=0.0;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*3];

points_no=55; x_dim=0.2; y_dim=0.15; dx=0.4; dy=0.5;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*4];

data=[data class]; %se adauga ca ultima coloana clasa de apartenenta/id_cluster
%--------------------------------------------------------------------------
function [data, no_vowels] = generare_clustere_vocale(atr, sel)
% Extract clusters from the file 'DB.mat' with statistic parametrs, 
% according to the vowels.
% The returned variabile no_vowels will specify how many valide vowels was founded 
    if nargin<2, sel='all'; end
    if nargin<1, atr=[6, 9] ; end %implicit F1 and F2
        
    if lower(sel)=='f', istart=1; istep=2; end
    if lower(sel)=='m', istart=2; istep=2; end
    if strcmp(lower(sel),'all'), istart=1; istep=1; end
 
    load DB
    
%    DB_all=DB{1}; %INCORECT because first emotional state(joy apear two times) 
    DB_all=DB{istart}; istart=istart+istep;
    for i=istart:istep:8   %all states(1:8), only female 1:2:8, only male 2:2:8
        for k=1:length(DB{i}),  %all the vowels
            for j=1:length(DB_all),
                if strcmp(DB{i}{k}{1}, DB_all{j}{1}),
                    DB_all{j}{2}=DB_all{j}{2}+DB{i}{k}{2};
                    for m=3:length(DB{i}{k})
                        DB_all{j}{m} = [DB_all{j}{m} DB{i}{k}{m}];
    end;end;end;end;end

    %establish the number of valide vowels (neglect NaN values for each vowel/phonem
    no_vowels = 0;
    for j=1:length(DB_all)
        %valid = find(~isnan(DB_all{j}{6}) & ~isnan(DB_all{j}{9}) );
        valid = ones(1,DB_all{j}{2});
        for k=1:length(atr)
            valid = valid & ~isnan(DB_all{j}{atr(k)}); %all values must be different of NaN
        end
        valid = find(valid);
        
        %no_vowels = [no_vowels  no_vowels(end)+DB_all{j}{2}];
        no_vowels = [no_vowels  no_vowels(end)+length(valid)];
    end

    %extract only the atributes from the list 'atr' for the construction of data set
    data = zeros(no_vowels(end),length(atr)+1);
    for j=1:length(DB_all)
        valid = ones(1,DB_all{j}{2});
        for k=1:length(atr)
            valid = valid & ~isnan(DB_all{j}{atr(k)}); %all values must be different of NaN
        end
        valid = find(valid);
        for k=1:length(atr)      
            data(no_vowels(j)+1:no_vowels(j+1), k) = DB_all{j}{atr(k)}(valid)';
        end
        data(no_vowels(j)+1:no_vowels(j+1), length(atr)+1) = j; %class - on last column
    end
%--------------------------------------------------------------------------
function [data, no_values] = generare_clustere_emotii(atr, sel, tip_parcurgere);
% Extract clusters from the file 'DB.mat' with statistic parametrs, 
% according to the emotions.
% The returned variabile no_values will specify how many valide values was founded 
%    sel = 'f', 'm', 'all' : gender of speaker female, male or all
%    tip_parcurgere=1 : read data according to the emotions
%    tip_parcurgere=2 : read data according to the phonem/vowel
if nargin<2, sel='all'; end
if nargin<1, atr=[6, 9] ; end %implicit F1 and F2
if nargin<3, tip_parcurgere=2; end
        
if lower(sel(1))=='f', istart=1; istep=2; end
if lower(sel(1))=='m', istart=2; istep=2; end
if strcmp(lower(sel),'all'), istart=1; istep=1; end

load DB
vowels_list = zeros(length(DB{istart}), length(DB{istart}{1}{1}) );
for j=1:length(DB{istart}),  %all vowels      
    vowels_list(j,:) = DB{istart}{j}{1};
end
    
data = [];

if tip_parcurgere==1,
    %read data in the order of the emotions
    for i=istart:istep:8   %all states(1:8), only female 1:2:8, only male 2:2:8
%        for j=7:7 %only a vowel 'e' 'i' 'a' 'a+' 'u' 'a-' 'o'
        for j=1:length(DB{i}),  %toate vocalele
           %compute the number of valid values (neglect NaN values for each vowel/phonem
            valid = ones(1,DB{i}{j}{2});
            for k=1:length(atr)
                valid = valid & ~isnan(DB{i}{j}{atr(k)}); %all values must be different of NaN
            end
            valid = find(valid);
 
            %extract only valide data
            data_crt = zeros(length(valid), length(atr)+2);
            for k=1:length(atr)
                data_crt(:,k) =  DB{i}{j}{atr(k)}(valid)';
            end

            id_vowel=0;
            for k=1:length(DB{istart}),  %all vowels   
                if strcmp(char(vowels_list(k,:)), DB{i}{j}{1})==1, id_vowel=k; break; end
            end
            data_crt(:, length(atr)+1) = i;        %type_emotion - penultima column
            data_crt(:, length(atr)+2) = id_vowel; %type_vowel - last column
            data = [data; data_crt];
            
            %disp(sprintf('%s %d %d %d',DB{i}{j}{1}, length(valid), i, id_vowel));
        end
    end
    no_values = [0 find(diff(data(:,end-1)))' size(data,1)];
end   
if tip_parcurgere==2,
    %read data in the order of vowels (echivalent with function 'generare_clustere_vocale')
    icrt=istart;
%    for j=1:1,
    for j=1:length(DB{icrt}),  %all vowels    
        for i=istart:istep:8   %all states(1:8), only female 1:2:8, only male 2:2:8
            id_vowel=0;
            for m=1:length(DB{istart}),  %all vowels   
                if strcmp(char(vowels_list(j,:)), DB{i}{m}{1})==0, continue; end
                id_vowel=m; 

                %establish the number of valide vowels (neglect NaN values for each vowel/phonem
                valid = ones(1,DB{i}{m}{2});
                for k=1:length(atr)
                    valid = valid & ~isnan(DB{i}{m}{atr(k)}); %toate valorile trebuie sa nu fie NaN
                end
                valid = find(valid);
     
                %extract only valide data
                data_crt = zeros(length(valid), length(atr)+2);
                for k=1:length(atr)
                    data_crt(:,k) =  DB{i}{m}{atr(k)}(valid)';
                end
        
                data_crt(:, length(atr)+1) = i; %type_emotion - penultima column
                data_crt(:, length(atr)+2) = j; %type_vowel - last column
                data = [data; data_crt];
            
                disp(sprintf('%s %d %d %d',DB{i}{m}{1}, length(valid), i, j));
            end
        end
        icrt=icrt+istart;
    end
    no_values = [0 find(diff(data(:,end)))' size(data,1)];
end
%--------------------------------------------------------------------------
