function test_SVM_emotions
% Parameters: mean_F0:3 mean_F1-F4:6,9,12,15 std_F0:4 std_F1-F4:7,10,13,16 median_F0:5 median_F1-F4:8,11,14,17
%             jitter: 18(v1),19(v2) shimmer:20
%             21:32 MFCC mean, 33:44 MFCC std, 45:56 delta MFCC, 57:68 delta delta MFCC
%             68:89 LPCC, 90:104 LPC, 105 prediction error, 106-119 PARCOR, 120:133 LAR, 134:148 AC
%             149:160 Lyap_MFCC, 161:181 Lyap_LPCC, 182:193 Energy, 194:205 Lyap_Energy
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE
%--------------------------------------------------------------------------

n_iter=3;

%atr=[3,4,5]; %F0 mean+std+median
%atr=[6,9,12,15]; %formants F1-F4
%atr=[3 6 9]; %F0,F1,F2
%atr=[18,19,20]; %jiter+shimmer
%atr=[3:20]; %list of all prosodic 
%atr=[21:56]; %MFCC mean 12 coef + std + delta

%atr=[69:89];   %LPCC 
%atr=[90:104];  %LPC 
%atr=[106:119]; %PARCOR 
%atr=[120:133]; %LAR 
%atr=[134:147]; %AC 

%atr=[149:160]; %Lyap_MFCC
%atr=[161:181]; %Lyap_LPCC 21 coef
atr=[21:32];

tip_parcurgere = 1; %1:classify acording to the emotion, 2:gender
tip_sex = 'all';    %'m'-male, 'f'-female, 'all'-all speakers
%---------------------------------
%Running parameters of SVM
percent = 0.75; %percent of date from training/testing data sets

%        'linear'      Linear kernel or dot product
%        'quadratic'   Quadratic kernel
%        'polynomial'  Polynomial kernel (default order 3)
%        'rbf'         Gaussian Radial Basis Function kernel
%        'mlp'         Multilayer Perceptron kernel (default scale 1)
kernel = 5;

%---------------------------------
% Generate the training and testing data sets
if exist('DB.mat','file')==2,
%    [data, no_values] = generare_clustere_vocale(atr,'f');
    [data, no_values] = generare_clustere_emotii(atr, tip_sex, tip_parcurgere);
    if tip_parcurgere==1, data = data(:,1:end-1); end %keep the class id penultima column (id_emotie)
    if tip_parcurgere==2, data = [data(:,1:end-2) data(:,end)]; end %keep the class id as last column (id_vocala)
    
    if tip_parcurgere==1 & strcmp(tip_sex,'all')==1,
        % Emotional states 1-'BF',2-'BM',3-'FF',4-'FM',5-'TF',6-'TM',7-'NF',8-'NM' become
        % only 4 emotional states 1-'B', 3-'F', 5-'T', 7-N' (B/joy,F/Fury,T/Sadness, N/neutral tone
        poz = find(mod(data(:,end),2)==0);
        data(poz,end) = data(poz,end)-1;
        %dim_classes = [0 find(diff(data(:,end)))' size(data,1)];
    end        
else
    disp('Baza de date cu valori formantice F0, F1,...,F4 nu a fost gasita');
    data = generare_clustere;
    no_values = [0 find(diff(data(:,end)))' size(data,1)];
end    

disp(['Set atribute:' sprintf('%d ', atr)]);

err_swm = zeros(n_iter,4); err_globala_svm = zeros(n_iter,1);

prag = round(diff(no_values)*percent); %used to select training/testing data sets
dim_test = diff(no_values)-prag; 
if strcmp(tip_sex,'all')==1, dim_test = dim_test(1:2:end)+dim_test(2:2:end); end

for i = 1 : n_iter
    data_a = []; %data_a = zeros(sum(prag),3);
    data_t = []; %data_t = zeros(size(data,1)-sum(prag),3);
    mark_all = [];
    for j = 1 : length(no_values)-1 %length(DB_all)
        %% Selection of first 'prag(j)' values in the training set, all other values are in testing data set
        %data_a = [data_a; data(no_values(j)+1:no_values(j)+prag(j), :)];
        %data_t = [data_t; data(no_values(j)+prag(j)+1:no_values(j+1), :)];

        %% Selection of first values in the training set
        %mark = zeros(1, no_values(j+1)-no_values(j)); mark(1:prag(j))=1; 
        %% Selection of last values in the training set
        %mark = zeros(1, no_values(j+1)-no_values(j)); mark(end-prag(j)+1:end)=1;
       
        % Random selection of values from training/testing data sets
        mark = ruleta(prag(j), no_values(j+1)-no_values(j));
        mark_all = [mark_all; mark'];
        data_a = [data_a; data(no_values(j)+find(mark), :)];
        data_t = [data_t; data(no_values(j)+find(~mark), :)];
    end
    err_svm(i,:) = rulare_svm(data, mark_all);
    %err_globala_svm(i) = sum(err_swm(i,:) .* dim_test)/sum(dim_test);
end
err_globala_svm = err_swm * dim_test'/sum(dim_test);


disp('-----------------');
if n_iter>1,
    disp(sprintf('Eroare minima: %.2f, eroare medie: %.2f, eroare maxima: %.2f', min(err_globala_svm), mean(err_globala_svm), max(err_globala_svm) ));
    %disp(['Eroare minima pentru fiecare clasa: ' sprintf(' %.2f,', min(err_svm)) ]);
    disp(['Eroare medie pentru fiecare clasa: ' sprintf(' %.2f,', mean(err_svm)) ]);
    %disp(['Eroare maxima pentru fiecare clasa: ' sprintf(' %.2f,', max(err_svm)) ]);
end

disp('gata');           
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
function mark = ruleta(n_sel, n_total)
% Selection o a number of 'n_sel' values from a total number of 'n_total' values/patterns
mark = zeros(1, n_total);
for i = 1:n_sel
    poz = fix(rand(1)*n_total)+1;
    while mark(poz)==1 | poz>n_total,
        poz = fix(rand(1)*n_total)+1;
    end
    mark(poz)=1;
end
%--------------------------------------------------------------------------
function data = generare_clustere
% Generate a number of 100 points distributed in the hole space, and after that
% generate 4 clusters with different dimension x_dim,y_dim and deplacements/ 
% translations dx,dy.  Each point/feature vector is described by (x,y) coordinates of points

data = rand(100,2);
class = zeros(100,1);
% First cluster
points_no=40; x_dim=0.2; y_dim=0.2; dx=0.8; dy=0.7;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*1];
% Second cluster
points_no=65; x_dim=0.15; y_dim=0.25; dx=0.05; dy=0.2;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*2];

points_no=70; x_dim=0.15; y_dim=0.2; dx=0.7; dy=0.0;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*3];

points_no=55; x_dim=0.2; y_dim=0.15; dx=0.4; dy=0.5;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*4];

data=[data class]; %se adauga ca ultima coloana clasa de apartenenta/id_cluster
%--------------------------------------------------------------------------
function [data, no_vowels] = generare_clustere_vocale(atr, sel)
% Extract clusters from the file 'DB.mat' with statistic parametrs, 
% according to the vowels.
% The returned variabile no_vowels will specify how many valide vowels was founded 
    if nargin<2, sel='all'; end
    if nargin<1, atr=[6, 9] ; end %implicit F1 and F2
        
    if lower(sel)=='f', istart=1; istep=2; end
    if lower(sel)=='m', istart=2; istep=2; end
    if strcmp(lower(sel),'all'), istart=1; istep=1; end
 
    load DB
    
%    DB_all=DB{1}; %INCORECT because first emotional state(joy apear two times) 
    DB_all=DB{istart}; istart=istart+istep;
    for i=istart:istep:8   %all states(1:8), only female 1:2:8, only male 2:2:8
        for k=1:length(DB{i}),  %all the vowels
            for j=1:length(DB_all),
                if strcmp(DB{i}{k}{1}, DB_all{j}{1}),
                    DB_all{j}{2}=DB_all{j}{2}+DB{i}{k}{2};
                    for m=3:length(DB{i}{k})
                        DB_all{j}{m} = [DB_all{j}{m} DB{i}{k}{m}];
    end;end;end;end;end

    %establish the number of valide vowels (neglect NaN values for each vowel/phonem
    no_vowels = 0;
    for j=1:length(DB_all)
        %valid = find(~isnan(DB_all{j}{6}) & ~isnan(DB_all{j}{9}) );
        valid = ones(1,DB_all{j}{2});
        for k=1:length(atr)
            valid = valid & ~isnan(DB_all{j}{atr(k)}); %all values must be different of NaN
        end
        valid = find(valid);
        
        %no_vowels = [no_vowels  no_vowels(end)+DB_all{j}{2}];
        no_vowels = [no_vowels  no_vowels(end)+length(valid)];
    end

    %extract only the atributes from the list 'atr' for the construction of data set
    data = zeros(no_vowels(end),length(atr)+1);
    for j=1:length(DB_all)
        valid = ones(1,DB_all{j}{2});
        for k=1:length(atr)
            valid = valid & ~isnan(DB_all{j}{atr(k)}); %all values must be different of NaN
        end
        valid = find(valid);
        for k=1:length(atr)      
            data(no_vowels(j)+1:no_vowels(j+1), k) = DB_all{j}{atr(k)}(valid)';
        end
        data(no_vowels(j)+1:no_vowels(j+1), length(atr)+1) = j; %class - on last column
    end
%--------------------------------------------------------------------------
function [data, no_values] = generare_clustere_emotii(atr, sel, tip_parcurgere);
% Extract clusters from the file 'DB.mat' with statistic parametrs, 
% according to the emotions.
% The returned variabile no_values will specify how many valide values was founded 
%    sel = 'f', 'm', 'all' : gender of speaker female, male or all
%    tip_parcurgere=1 : read data according to the emotions
%    tip_parcurgere=2 : read data according to the phonem/vowel
if nargin<2, sel='all'; end
if nargin<1, atr=[6, 9] ; end %implicit F1 and F2
if nargin<3, tip_parcurgere=2; end
        
if lower(sel(1))=='f', istart=1; istep=2; end
if lower(sel(1))=='m', istart=2; istep=2; end
if strcmp(lower(sel),'all'), istart=1; istep=1; end

load DB
vowels_list = zeros(length(DB{istart}), length(DB{istart}{1}{1}) );
for j=1:length(DB{istart}),  %all vowels      
    vowels_list(j,:) = DB{istart}{j}{1};
end
    
data = [];

if tip_parcurgere==1,
    %read data in the order of the emotions
    for i=istart:istep:8   %all states(1:8), only female 1:2:8, only male 2:2:8
%        for j=7:7 %only a vowel 'e' 'i' 'a' 'a+' 'u' 'a-' 'o'
        for j=1:length(DB{i}),  %toate vocalele
           %compute the number of valid values (neglect NaN values for each vowel/phonem
            valid = ones(1,DB{i}{j}{2});
            for k=1:length(atr)
                valid = valid & ~isnan(DB{i}{j}{atr(k)}); %all values must be different of NaN
            end
            valid = find(valid);
 
            %extract only valide data
            data_crt = zeros(length(valid), length(atr)+2);
            for k=1:length(atr)
                data_crt(:,k) =  DB{i}{j}{atr(k)}(valid)';
            end

            id_vowel=0;
            for k=1:length(DB{istart}),  %all vowels   
                if strcmp(char(vowels_list(k,:)), DB{i}{j}{1})==1, id_vowel=k; break; end
            end
            data_crt(:, length(atr)+1) = i;        %type_emotion - penultima column
            data_crt(:, length(atr)+2) = id_vowel; %type_vowel - last column
            data = [data; data_crt];
            
            %disp(sprintf('%s %d %d %d',DB{i}{j}{1}, length(valid), i, id_vowel));
        end
    end
    no_values = [0 find(diff(data(:,end-1)))' size(data,1)];
end   
if tip_parcurgere==2,
    %read data in the order of vowels (echivalent with function 'generare_clustere_vocale')
    icrt=istart;
%    for j=1:1,
    for j=1:length(DB{icrt}),  %all vowels    
        for i=istart:istep:8   %all states(1:8), only female 1:2:8, only male 2:2:8
            id_vowel=0;
            for m=1:length(DB{istart}),  %all vowels   
                if strcmp(char(vowels_list(j,:)), DB{i}{m}{1})==0, continue; end
                id_vowel=m; 

                %establish the number of valide vowels (neglect NaN values for each vowel/phonem
                valid = ones(1,DB{i}{m}{2});
                for k=1:length(atr)
                    valid = valid & ~isnan(DB{i}{m}{atr(k)}); %toate valorile trebuie sa nu fie NaN
                end
                valid = find(valid);
     
                %extract only valide data
                data_crt = zeros(length(valid), length(atr)+2);
                for k=1:length(atr)
                    data_crt(:,k) =  DB{i}{m}{atr(k)}(valid)';
                end
        
                data_crt(:, length(atr)+1) = i; %type_emotion - penultima column
                data_crt(:, length(atr)+2) = j; %type_vowel - last column
                data = [data; data_crt];
            
                disp(sprintf('%s %d %d %d',DB{i}{m}{1}, length(valid), i, j));
            end
        end
        icrt=icrt+istart;
    end
    no_values = [0 find(diff(data(:,end)))' size(data,1)];
end
%--------------------------------------------------------------------------
function err = rulare_svm(data, mark_all, kernel)
if nargin<3, kernel =1; end
switch(kernel)
    case 1
        KFUN='linear';
    case 2
        KFUN='quadratic';
    case 3
        KFUN='polynomial';
    case 4
        KFUN='rbf';
    case 5
        KFUN='mlp';
    otherwise
        KFUN='linear';
end
classes = 'BFTN'; %B/Joy F/Fury T/Sadness N/Neutral tone
rate = zeros(4,1);
n_err = 0;
for i=1:4
    id = i*2-1;
    groups = ismember(data(:,end),id);
    cp = classperf(groups);
    %svmStruct = svmtrain(data_a(:,1:end-1),ismember(data_a(:,end),id));
    %classes = svmclassify(svmStruct,data_t(:,1:end-1));
    %classperf(cp,classes,~mark_all);
    train = boolean(mark_all); 
    test=~train; 
    %svmStruct = svmtrain(data(train,1:end-1),ismember(data(train,end),id));
    %svmStruct = svmtrain(data(train,1:end-1),groups(train),'KERNEL_FUNCTION',KFUN);
    svmStruct = svmtrain(data(train,1:end-1),ismember(data(train,end),id),'KERNEL_FUNCTION',KFUN);
    
    classes = svmclassify(svmStruct,data(test,1:end-1));
    classperf(cp,classes,test);
    rate(i) = cp.CorrectRate;
    
end    
rate=rate*100; err=100-rate;
%disp(sprintf('SVM Rata de recunoastere: B %.2f |F %.2f |T %.2f|N %.2f',rate(1),rate(3),rate(3),rate(4)));
fprintf('SVM Eroare: B %.2f |F %.2f |T %.2f|N %.2f \n',err(1),err(2),err(3),err(4));