function test_fuzzy_c_means_vowels
%--------------------------------------------------------------------------
% Copyright(c) Zbancioc Marius-Dan, 2014, zmarius@etti.tuiasi.ro
% 1)Institute of Computer Science, Roumanian Academy, Iasi Branch, 
% 2)Technical University "Gheorghe Asachi" of Iasi, Romania
%--------------------------------------------------------------------------
% References:
%- M. Zbancioc, M. Feraru, The Analysis of the FCM and WKNN Algorithms Performance for the Emotional Corpus SROL, Advances in Electrical and Computer Engineering, Vol. 12, Issue 3, 2012, pp. 33  38, ISSN: 1582-7445, e-ISSN: 1844-7600, Digital Object Identifier: 10.4316/AECE.2012.03005
%- M. Feraru, M. Zbancioc, Speech Emotion Recognition for SROL Database using Weighted KNN algorithm, International Conference on Electronics, Computers and Artificial Intelligence (ECAI2013), June 27-29, 2013, Pitesti, Romania, ISBN: 978-1-4673-4935-2, Digital Object Identifier: 10.1109/ECAI.2013.6636189 2013 IEEE
%- M. Zbancioc, M. Feraru, A Study about MFCC Relevance in Emotion Classification for SROL Database, Proceedings of 4th International Symposium on Electrical and Electronics Engineering, ISEEE 2013, October, 11-13, 2013, Gala?i, Romnia, ISBN 978-1-4799-2441-7, IEEE Catalog Number CFP1393K-USB.
%- M. Zbancioc, M. Feraru, Using the Lyapunov Exponent from Cepstral Coefficients for Automatic Emotion Recognition, Proc of 8th IEEE International Conference on Electrical and Power Engineering  EPE 2014, 16-17 Oct., Ia?i, Romnia, ISSN: 978-1-4799-5848-1.
%- M. Feraru, M. Zbancioc, Emotion Recognition in Romanian Language using LPC Features, The 4th IEEE International Conference on E-Health and Bioengineering - EHB 2013, November 21-23, 2013, Ia?i, Romania, ISBN: 978-1-4799-2373-1/13/$31.00 2013 IEEE
%--------------------------------------------------------------------------

rulari_multiple=1; %0-FALSE, 1-TRUE
%mean_F0:3 mean_F1-F4:6,9,12,15 std_F0:4 std_F1-F4:7,10,13,16 median_F0:5 median_F1-F4:8,11,14,17
atr=[3:17]; %list of atributes
tip_parcurgere = 1; %1:classify acording to the emotion, 2:gender
tip_sex = 'all'; %'m'-male, 'f'-female, 'all'-all speakers
%---------------------------------
%Parameters Fuzzy-C-Means
n_clusters=7;

[DB DB_all] = construct_DB_all_emotions;

%---------------------------------
% Generate the training and testing data sets
if exist('DB.mat','file')==2,
%    [data, no_values] = generare_clustere_vocale(atr,'f');
    [data, no_values] = generare_clustere_emotii(atr, tip_sex, tip_parcurgere);
    if tip_parcurgere==1, data = data(:,1:end-1); end %keep the class id penultima column (id_emotie)
    if tip_parcurgere==2, data = [data(:,1:end-2) data(:,end)]; end %keep the class id as last column (id_vocala)
    
    if tip_parcurgere==1 & strcmp(tip_sex,'all')==1,
        % Emotional states 1-'BF',2-'BM',3-'FF',4-'FM',5-'TF',6-'TM',7-'NF',8-'NM' become
        % only 4 emotional states 1-'B', 3-'F', 5-'T', 7-N' (B/joy,F/Fury,T/Sadness, N/neutral tone
        poz = find(mod(data(:,end),2)==0);
        data(poz,end) = data(poz,end)-1;
    end        
else
    disp('Baza de date cu valori formantice F0, F1,...,F4 nu a fost gasita');
    data = generare_clustere;
    no_values = [0 find(diff(data(:,end)))' size(data,1)];
end    

disp(['Set atribute:' sprintf('%d ', atr)]);

if rulari_multiple==1, n_iter=50; else n_iter=1; end
err_min = zeros(1,n_iter);

%--------------------------------------------------------------------------
% if rulari_multiple==1,
%     n_iter=50; data_dim=size(data,2);
%     center_history = zeros(n_iter,n_clusters,data_dim);
%     figure(1); plot(data(:,1), data(:,2),'.'); hold on;
%     
%     for i=1:n_iter
%         [center,U,obj_fcn] = fcm(data, n_clusters);
%         plot([center(:,1)],[center(:,2)],'*','color','r'); %pause;    
%         center_history(i,:,:)=center;
%     end 
%     figure(1); hold off;
%     figure(2); plot([center_history(:,:,1)],[center_history(:,:,2)],'*','color','k');
% else
%     [center,U,obj_fcn] = fcm(data,n_clusters);
% end
% 
% maxU = max(U);
% % Find the data points with highest grade of membership in cluster 1
% index1 = find(U(1,:) == maxU);
% % Find the data points with highest grade of membership in cluster 2
% index2 = find(U(2,:) == maxU);
% 
% figure(3); plot(data(index1,1), data(index1,2),'sb'); hold on;
%            plot(data(index2,1), data(index2,2),'or');
%            % Plot the cluster centers
%            plot([center([1 2],1)],[center([1 2],2)],'*','color','k'); hold off;
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------
% Running og algorithm fuzzy_c_means
if rulari_multiple==1,
    n_iter=50; data_dim=size(data,2);
    center = zeros(n_clusters,data_dim);
    center_history = zeros(n_iter,n_clusters,data_dim);
%    figure(1); plot(data(:,1), data(:,2),'.'); hold on;
    figure(1); plot(data(:,1), data(:,2),'o'); hold on;
    
    for i=1:n_iter
        [trace_center,U,obj_fcn] = fuzzy_c_means_CR(data,n_clusters,0,1);
        %line([center(:,:,1)],[center(:,:,2)],'marker','.','color','k'); %pause;
        line([trace_center(:,:,1)],[trace_center(:,:,2)],'LineStyle','-','color','k'); %pause;
        center = trace_center(end,:,:);
        plot([center(end,:,1)],[center(end,:,2)],'*','color','r'); pause(0);    
        center_history(i,:,:)=center(end,:,:);
    end     
    figure(1); hold off;
    figure(2); plot(data(:,1), data(:,2),'.','MarkerSize',4); hold on;
               plot([center_history(:,:,1)],[center_history(:,:,2)],'*','color','r'); hold off;
else
    [trace_center,U,obj_fcn] = fuzzy_c_means_CR(data,n_clusters,0,1);
    center = trace_center(end,:,:);
end    
    
maxU = max(U);
if n_clusters==2
    % Find the data points with highest grade of membership in cluster 1
    index1 = find(U(1,:) == maxU);
    % Find the data points with highest grade of membership in cluster 2
    index2 = find(U(2,:) == maxU);
    
        figure(3); plot(data(index1,1), data(index1,2),'sb'); hold on;
               plot(data(index2,1), data(index2,2),'or');
               % Plot the trace of cluster centers
else
    markers = 'so*x+Dphv^<>'; n_markers = length(markers);
    colors = 'brmbgc'; n_colors = length(colors);
    cluster_FCM = zeros( size(data,1), 1);
    for i=1:n_clusters
        index_i = find(U(i,:) == maxU);
        cluster_FCM(index_i)=i;
        crt_color = mod(i,n_colors); if crt_color==0, crt_color=n_colors; end
        crt_marker = mod(i,n_markers); if crt_marker==0, crt_marker=n_markers; end
        crt_color = colors(crt_color); crt_marker = markers(crt_marker);
        figure(3); plot(data(index_i,1), data(index_i,2),[crt_color crt_marker]); hold on;
    end
end        
figure(3); line([trace_center(:,:,1)],[trace_center(:,:,2)],'marker','.','color','k');
           % Plot the final position of cluster centers
           if rulari_multiple==1,
               plot([center(end,:,1)],[center(end,:,2)],'p','color','k','MarkerFaceColor','g','MarkerSize',10); hold off;
           else
               plot([center(1,:,1)],[center(1,:,2)],'p','color','k','MarkerFaceColor','g','MarkerSize',10); hold off;
           end
figure(5); bar(hist(U(1,:),10))           
figure(4); subplot(211); plot(obj_fcn); grid; title('Objective function - obj_{fcn}');
           subplot(212); plot(diff(obj_fcn)); grid; title('Improvement - diff(obj_{fcn})');

err_matrix = zeros(n_clusters,n_clusters);
for i=1:n_clusters
    for j=1:n_clusters
        err_matrix(i,j) = length(find(data(:,end)==i & cluster_FCM==j));
    end
end
err_vowel = zeros(n_clusters,1);
for i=1:n_clusters
    err_vowel(i) = 1-max(err_matrix(i,:))/sum(err_matrix(i,:));
    disp(sprintf('Vocala: %s Eroare: %.2f Corect: %.2f',DB_all{i}{1}, err_vowel(i)*100, (1-err_vowel(i))*100  ));
end    
disp(sprintf('EroareTotala: %.2f ',(1-sum(max(err_matrix'))/sum(sum(err_matrix)))*100  ));

disp('gata');           
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
function mark = ruleta(n_sel, n_total)
% Selection o a number of 'n_sel' values from a total number of 'n_total' values/patterns
mark = zeros(1, n_total);
for i = 1:n_sel
    poz = fix(rand(1)*n_total)+1;
    while mark(poz)==1 | poz>n_total,
        poz = fix(rand(1)*n_total)+1;
    end
    mark(poz)=1;
end    
%--------------------------------------------------------------------------
function data = generare_clustere
% Generate a number of 100 points distributed in the hole space, and after that
% generate 4 clusters with different dimension x_dim,y_dim and deplacements/ 
% translations dx,dy.  Each point/feature vector is described by (x,y) coordinates of points

data = rand(100,2);
class = zeros(100,1);
% First cluster
points_no=40; x_dim=0.2; y_dim=0.2; dx=0.8; dy=0.7;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*1];
% Second cluster
points_no=65; x_dim=0.15; y_dim=0.25; dx=0.05; dy=0.2;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*2];

points_no=70; x_dim=0.15; y_dim=0.2; dx=0.7; dy=0.0;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*3];

points_no=55; x_dim=0.2; y_dim=0.15; dx=0.4; dy=0.5;
cluster = rand(points_no,2)*[x_dim 0; 0 y_dim] + ones(points_no,2)*[dx 0; 0 dy];
data = [data; cluster];
class = [class; ones(points_no,1)*4];

data=[data class]; %last column is the membership function/id_cluster
%--------------------------------------------------------------------------
function [data, no_vowels] = generare_clustere_vocale(atr, sel)
% Extract clusters from the file 'DB.mat' with statistic parametrs, 
% according to the vowels.
% The returned variabile no_vowels will specify how many valide vowels was founded 
    if nargin<2, sel='all'; end
    if nargin<1, atr=[6, 9] ; end %implicit F1 and F2
        
    if lower(sel)=='f', istart=1; istep=2; end
    if lower(sel)=='m', istart=2; istep=2; end
    if strcmp(lower(sel),'all'), istart=1; istep=1; end
 
    load DB
    
%    DB_all=DB{1}; %INCORECT because first emotional state(joy apear two times) 
    DB_all=DB{istart}; istart=istart+istep;
    for i=istart:istep:8   %all states(1:8), only female 1:2:8, only male 2:2:8
        for k=1:length(DB{i}),  %all the vowels
            for j=1:length(DB_all),
                if strcmp(DB{i}{k}{1}, DB_all{j}{1}),
                    DB_all{j}{2}=DB_all{j}{2}+DB{i}{k}{2};
                    for m=3:length(DB{i}{k})
                        DB_all{j}{m} = [DB_all{j}{m} DB{i}{k}{m}];
    end;end;end;end;end

    %establish the number of valide vowels (neglect NaN values for each vowel/phonem
    no_vowels = 0;
    for j=1:length(DB_all)
        %valid = find(~isnan(DB_all{j}{6}) & ~isnan(DB_all{j}{9}) );
        valid = ones(1,DB_all{j}{2});
        for k=1:length(atr)
            valid = valid & ~isnan(DB_all{j}{atr(k)}); %all values must be different of NaN
        end
        valid = find(valid);
        
        %no_vowels = [no_vowels  no_vowels(end)+DB_all{j}{2}];
        no_vowels = [no_vowels  no_vowels(end)+length(valid)];
    end

    %extract only the atributes from the list 'atr' for the construction of data set
    data = zeros(no_vowels(end),length(atr)+1);
    for j=1:length(DB_all)
        valid = ones(1,DB_all{j}{2});
        for k=1:length(atr)
            valid = valid & ~isnan(DB_all{j}{atr(k)}); %all values must be different of NaN
        end
        valid = find(valid);
        for k=1:length(atr)      
            data(no_vowels(j)+1:no_vowels(j+1), k) = DB_all{j}{atr(k)}(valid)';
        end
        data(no_vowels(j)+1:no_vowels(j+1), length(atr)+1) = j; %class - on last column
    end
%--------------------------------------------------------------------------
function [data, no_values] = generare_clustere_emotii(atr, sel, tip_parcurgere);
% Extract clusters from the file 'DB.mat' with statistic parametrs, 
% according to the emotions.
% The returned variabile no_values will specify how many valide values was founded 
%    sel = 'f', 'm', 'all' : gender of speaker female, male or all
%    tip_parcurgere=1 : read data according to the emotions
%    tip_parcurgere=2 : read data according to the phonem/vowel
if nargin<2, sel='all'; end
if nargin<1, atr=[6, 9] ; end %implicit F1 and F2
if nargin<3, tip_parcurgere=2; end
        
if lower(sel(1))=='f', istart=1; istep=2; end
if lower(sel(1))=='m', istart=2; istep=2; end
if strcmp(lower(sel),'all'), istart=1; istep=1; end

load DB
vowels_list = zeros(length(DB{istart}), length(DB{istart}{1}{1}) );
for j=1:length(DB{istart}),  %all vowels    
    vowels_list(j,:) = DB{istart}{j}{1};
end
    
data = [];

if tip_parcurgere==1,
    %read data in the order of the emotions
    for i=istart:istep:8   %all states(1:8), only female 1:2:8, only male 2:2:8
%        for j=7:7 %only a vowel 'e' 'i' 'a' 'a+' 'u' 'a-' 'o'
        for j=1:length(DB{i}),  %all the vowels
           %compute the number of valid values (neglect NaN values for each vowel/phonem
            valid = ones(1,DB{i}{j}{2});
            for k=1:length(atr)
                valid = valid & ~isnan(DB{i}{j}{atr(k)}); %all values must be different of NaN
            end
            valid = find(valid);
 
            %extract only valide data
            data_crt = zeros(length(valid), length(atr)+2);
            for k=1:length(atr)
                data_crt(:,k) =  DB{i}{j}{atr(k)}(valid)';
            end

            id_vowel=0;
            for k=1:length(DB{istart}),  %all vowels
                if strcmp(char(vowels_list(k,:)), DB{i}{j}{1})==1, id_vowel=k; break; end
            end
            data_crt(:, length(atr)+1) = i;        %type_emotion - penultima column
            data_crt(:, length(atr)+2) = id_vowel; %type_vowel - last column
            data = [data; data_crt];
            
            %disp(sprintf('%s %d %d %d',DB{i}{j}{1}, length(valid), i, id_vowel));
        end
    end
    no_values = [0 find(diff(data(:,end-1)))' size(data,1)];
end   
if tip_parcurgere==2,
    %read data in the order of vowels (echivalent with function 'generare_clustere_vocale')
    icrt=istart;
%    for j=1:1,
    for j=1:length(DB{icrt}),  %all vowels    
        for i=istart:istep:8   %all the states(1:8), only female 1:2:8, only male 2:2:8
            id_vowel=0;
            for m=1:length(DB{istart}),  %all vowels    
                if strcmp(char(vowels_list(j,:)), DB{i}{m}{1})==0, continue; end
                id_vowel=m; 

                %establish the number of valide vowels (neglect NaN values for each vowel/phonem
                valid = ones(1,DB{i}{m}{2});
                for k=1:length(atr)
                    valid = valid & ~isnan(DB{i}{m}{atr(k)}); %all values must be different of NaN
                end
                valid = find(valid);
     
                %extract only valide data
                data_crt = zeros(length(valid), length(atr)+2);
                for k=1:length(atr)
                    data_crt(:,k) =  DB{i}{m}{atr(k)}(valid)';
                end
        
                data_crt(:, length(atr)+1) = i; %type_emotion - penultima column
                data_crt(:, length(atr)+2) = j; %type_vowel - last column
                data = [data; data_crt];
            
                disp(sprintf('%s %d %d %d',DB{i}{m}{1}, length(valid), i, j));
            end
        end
        icrt=icrt+istart;
    end
    no_values = [0 find(diff(data(:,end)))' size(data,1)];
end
%--------------------------------------------------------------------------
